@@ -33,6 +33,7 @@ from typing import (
 from collections import namedtuple
 
 from sqlfluff.core.cached_property import cached_property
+from sqlfluff.core.config import FluffConfig
 
 from sqlfluff.core.linter import LintedFile, NoQaDirective
 from sqlfluff.core.parser import BaseSegment, PositionMarker, RawSegment
@@ -64,7 +65,7 @@ class LintResult:
 
     Args:
         anchor (:obj:`BaseSegment`, optional): A segment which represents
-            the *position* of the a problem. NB: Each fix will also hold
+            the *position* of the problem. NB: Each fix will also hold
             its own reference to position, so this position is mostly for
             alerting the user to where the *problem* is.
         fixes (:obj:`list` of :obj:`LintFix`, optional): An array of any
@@ -235,8 +236,9 @@ class LintFix:
                 detail = f"create:{new_detail!r}"
         else:
             detail = ""  # pragma: no cover TODO?
-        return "<LintFix: {} @{} {}>".format(
-            self.edit_type, self.anchor.pos_marker, detail
+        return (
+            f"<LintFix: {self.edit_type} {self.anchor.get_type()}"
+            f"@{self.anchor.pos_marker} {detail}>"
         )
 
     def __eq__(self, other):
@@ -300,7 +302,7 @@ class LintFix:
         #    character positions surrounding the insertion point (**NOT** the
         #    whole anchor segment, because we're not *touching* the anchor
         #    segment, we're inserting **RELATIVE** to it.
-        assert self.anchor.pos_marker
+        assert self.anchor.pos_marker, f"Anchor missing position marker: {self.anchor}"
         anchor_slice = self.anchor.pos_marker.templated_slice
         templated_slices = [anchor_slice]
 
@@ -482,6 +484,7 @@ class BaseRule:
         templated_file: Optional["TemplatedFile"],
         ignore_mask: List[NoQaDirective],
         fname: Optional[str],
+        config: FluffConfig,
     ) -> Tuple[List[SQLLintError], Tuple[RawSegment, ...], List[LintFix], Any]:
         """Run the rule on a given tree.
 
@@ -495,6 +498,7 @@ class BaseRule:
             templated_file=templated_file,
             path=pathlib.Path(fname) if fname else None,
             segment=tree,
+            config=config,
         )
         vs: List[SQLLintError] = []
         fixes: List[LintFix] = []
@@ -545,7 +549,7 @@ class BaseRule:
                 memory = res.memory
                 self._adjust_anchors_for_fixes(context, res)
                 self._process_lint_result(
-                    res, templated_file, ignore_mask, new_lerrs, new_fixes
+                    res, templated_file, ignore_mask, new_lerrs, new_fixes, tree
                 )
             elif isinstance(res, list) and all(
                 isinstance(elem, LintResult) for elem in res
@@ -556,7 +560,7 @@ class BaseRule:
                 for elem in res:
                     self._adjust_anchors_for_fixes(context, elem)
                     self._process_lint_result(
-                        elem, templated_file, ignore_mask, new_lerrs, new_fixes
+                        elem, templated_file, ignore_mask, new_lerrs, new_fixes, tree
                     )
             else:  # pragma: no cover
                 raise TypeError(
@@ -582,13 +586,43 @@ class BaseRule:
         pass
 
     def _process_lint_result(
-        self, res, templated_file, ignore_mask, new_lerrs, new_fixes
+        self, res, templated_file, ignore_mask, new_lerrs, new_fixes, root
     ):
         self.discard_unsafe_fixes(res, templated_file)
         lerr = res.to_linting_error(rule=self)
         ignored = False
         if lerr:
-            if ignore_mask:
+            # Check whether this should be filtered out for being unparsable.
+            # To do that we check the parents of the anchors (of the violation
+            # and fixes) against the filter in the crawler.
+            # NOTE: We use `.passes_filter` here to do the test for unparsable
+            # to avoid duplicating code because that test is already implemented
+            # there.
+            anchors = [lerr.segment] + [fix.anchor for fix in lerr.fixes]
+            for anchor in anchors:
+                if not self.crawl_behaviour.passes_filter(anchor):  # pragma: no cover
+                    # NOTE: This clause is untested, because it's a hard to produce
+                    # edge case. The latter clause is much more likely.
+                    linter_logger.info(
+                        "Fix skipped due to anchor not passing filter: %s", anchor
+                    )
+                    lerr = None
+                    ignored = True
+                    break
+                parent_stack = root.path_to(anchor)
+                if not all(
+                    self.crawl_behaviour.passes_filter(ps.segment)
+                    for ps in parent_stack
+                ):
+                    linter_logger.info(
+                        "Fix skipped due to parent of anchor not passing filter: %s",
+                        [ps.segment for ps in parent_stack],
+                    )
+                    lerr = None
+                    ignored = True
+                    break
+
+            if lerr and ignore_mask:
                 filtered = LintedFile.ignore_masked_violations([lerr], ignore_mask)
                 if not filtered:
                     lerr = None
@@ -655,7 +689,7 @@ class BaseRule:
         if seg.raw_upper in [
             elem[1] for elem in target_tuples if elem[0] == "raw_upper"
         ]:
-            return True
+            return True  # pragma: no cover
         elif seg.is_type(*[elem[1] for elem in target_tuples if elem[0] == "type"]):
             return True
         # For parent type checks, there's a higher risk of getting an incorrect
@@ -722,7 +756,7 @@ class BaseRule:
             return
 
     @classmethod
-    def _adjust_anchors_for_fixes(cls, context, lint_result):
+    def _adjust_anchors_for_fixes(cls, context, lint_result) -> None:
         """Makes simple fixes to the anchor position for fixes.
 
         Some rules return fixes where the anchor is too low in the tree. These
@@ -834,8 +868,8 @@ class RuleSet:
 
     The code for the rule will be parsed from the name, the description
     from the docstring. The eval function is assumed that it will be
-    overriden by the subclass, and the parent class raises an error on
-    this function if not overriden.
+    overridden by the subclass, and the parent class raises an error on
+    this function if not overridden.
 
     """
 
